# -*- coding: utf-8 -*-
import fnmatch
import os

"""Ensemble de fonctions sur fichiers / repertoire"""


def filerecorder(doc, fp, b=False):
    """ Enregistrement d'un fichier

    Le chemin du filpath sera construit si besoin

    :param  doc: données à enregistrer
    :param str fp: filepath
    :param boolean b: données en byte ?, optional
    :return:
    """
    makedirs(dirparent(fp))  # Check dir exists

    with open(fp, 'wb') if b else open(fp, 'w', encoding='utf-8') as f:
        f.write(doc)


def fileloader(fp, b=False):
    """ Chargement d'un fichier

    :param fp: filepath du fichier recherché
    :param bool b: données en byte(False)  ? optional
    :return: document
    """
    with open(fp, 'rb') if b else open(fp, 'r') as f:
        return f.read()


def dirproject():
    """ Répertoire d'execution """
    return os.getcwd()


def dirparent(path):
    """  Renvoie du repertoire parent

    :param str path: repertoire
    :rtype: str

    """
    return os.path.dirname(os.path.realpath(path))


def dircurrent(source=None):
    """Répertoire pour le fichier en cours """
    return dirparent(source or __file__)


def dirparser(directory, pattern="*"):
    """Récupération des fichiers d'un répertoire

    :param str directory: repertoire
    :param str pattern: '*' pour tous type de fichier par défaut

    :Exemple:
        >>> directory = '/home/user/Documents'
        >>> pattern='*.txt'
        >>> for filename, path_file in dirparser(directory, pattern):
        ...    print(path_file)
        'home/user/Documents/fichier.txt'
        'home/user/Documents/autre_fichier.txt'
    """

    for root, dirs, files in os.walk(directory):
        for filename in files:
            if fnmatch.fnmatch(filename, pattern):
                yield filename, os.path.join(root, filename)


def path_build(directory, ps_complement):
    """ Construction d'un pathfile

    :param str directory: repertoire
    :param str ps_complement: complement permettant de generer le chemin
    :rtype: str

    :Exemple:
        >>> path = 'home/user/Documents'
        >>> path_build(path, '../other_dir')
        'home/user/Documents/other_dir'
    """
    return os.path.abspath(os.path.join(directory, ps_complement))


def file_ext(ps_file):
    """ Retrourne l'extension d'un fichier

    :param ps_file:
    :return: Extension de fichier
    """
    return os.path.splitext(ps_file)[1]


def file_ext_less(ps_file):
    """ Retrourne le fichier sans extension

    :param ps_file:
    :return: Extension de fichier

    Example
    ----------
        >>> f = 'filename.ext'
        >>> file_ext_less(f)
        filename
    """
    return os.path.splitext(ps_file)[1]


def file_exists(fp):
    """Vérifie l'existance d'un fichier

    :param str fp: filepath
    :rtype bool:
    """

    return os.path.exists(fp)


def makedirs(path):
    """ Création du répertoire données

    :param path: chemin du répertoire à créer
    :rtype bool:

    """

    if not file_exists(path):
        os.makedirs(path)
        return file_exists(path)

    return True


def remove_file(p):
    """ Suppression d'un fichier si existant

    :param str p: chemin complet du fichier à supprimer
    """
    if file_exists(p):
        os.remove(p)


def clean_directory(directory, pattern='*'):
    """ Supprimes tous les élements d'un repertoire

    :param str directory: chemin du repertoire
    :param string pattern: patter des fichier à supprimer (filtre)
    :return int: nombre de fichier supprimer

    """
    i_count = 0

    for filename, path_file in dirparser(directory, pattern):
        remove_file(path_file)
        i_count += 1
    return i_count



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Liste de fonction standard reutilisable

.. author: Dreamgeeker
"""

__author__ = "Ketsia LENTIN"
__copyright__ = "Copyright 2020, Couleur West IT"
__credits__ = "Ketsia LENTIN"
__license__ = "MIT"
__version__ = "1.2.5"
__email__ = "contact@couleurwest-it"
__status__ = "Production"
__docformat__ = 'reStructuredText'



# -*- coding: utf-8 -*-
from urllib.parse import unquote
from cerberus.validator import Validator, schema_registry

from . import dtemng
from . import tools
from .logmng import (CTracker, CError)
from .cfgmng import CFGBases

schema_registry.add('email scheme', {'email': {'type': 'string', 'regex': tools.RGX_EMAIL}})
schema_registry.add('password scheme', {'password': {'type': 'string', 'regex': tools.RGX_PWD}})
schema_registry.add('phone scheme', {'phone': {'type': 'string', 'regex': tools.RGX_PHONE}})


class Validata(Validator):
    """
    Validators
    """
    _normalisator = CFGBases.normalizor()
    _validators = CFGBases.validator()
    liste_categorie = []

    def __init__(self, scheme, *args, **kwargs):
        dic = self._validators.get(scheme)
        dic['creation_date'] = {'type': 'integer', 'default_setter': 'utcnow'}

        super().__init__(dic, *args, **kwargs)

    def _normalize_default_setter_utcnow(self, document):
        """Initialisateur date par defaut"""
        return dtemng.utcnow_ts()

    def _normalize_coerce_utcts(self, value):
        """Transforme une date valide en timestamp"""
        try:
            dte = dtemng.isotodate(value)  # str -> date
            dte = dtemng.datetime_from_local_to_utc(dte)  # local -> utc
        except:
            return None
        else:
            return dtemng.dtets(dte)  # dte ->timestamp

    def _normalize_coerce_int(self, value):
        """Transforme un nombre au format chaine en entier"""
        return int(value)

    def _normalize_coerce_clean_string(self, value):
        """Nettoyeur de chaine"""
        return tools.clean_space(value)

    def _normalize_coerce_event(self, value):
        """Validation de date au format '%d.%m.%Y %H:%M'"""
        try:
            dte = dtemng.strdate(value, '%d.%m.%Y %H:%M')
            dte = dtemng.dtets(dte)

            return dte
        except Exception as ex:
            CTracker.exception_tracking(ex, 'validata.validata._normalize_coerce_event')
            return None

    def _check_with_validate(self, field, value):
        if dtemng.datestr(value) is None:
            self._error(field, "Date not valid")

    def _check_with_categorie(self, field, value):
        """Verifie si l'element figure dans une liste à configurer"""
        if value not in self.liste_categorie:
            self._error(field, "value not valid")

    def _check_with_url(self, field, value):
        from dreamtools import features

        if value and not features.test_http_link(value):
            self._error(field, "lient url non valid")

    def _check_with_email(self, field, value):
        from verify_email import verify_email

        if not verify_email(value, verify=False):
           self._error(field, "Adresse non valide")

    def validation(self, document, *args, **kwargs):
        """

        :param document:
        :param args:
        :param kwargs:
        :return:
        """
        if self.validate(document, *args, **kwargs):
            return self.validated(document)
        return None

    def normalisation(self, document):
        """

        :param document:
        :param args:
        :param kwargs:
        :return:
        """
        return Validator(self._normalisator).normalized(document)

    @classmethod
    def check_post_data(cls, data, form_ref):
        """ verification donnees formulaire recu

        :param dict data: formulaire de données
        :param str form_ref: reference validateur

        :return: formulaire traité
        """

        def fn():
            """
            function interne
            :return:
            """
            o = cls(form_ref, purge_unknown=True)
            d = {k: unquote(tools.clean_space(v)) for k, v in data.items()}

            d = o.validation(d)
            if d:
                return d

            d = o.normalisation(o.errors)
            raise CError(','.join(d.keys()), 400, "VALIDATA.CHECK_POST_DATA")

        reponce = CTracker.fntracker(fn, 'VALIDATA.CHECK_POST_DATA')

        if not reponce.ok and type(reponce.message).__name__ == 'dict':
            reponce.message = reponce.message.values()

        return reponce
